'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var React = require('react');
var Koa = require('koa');
var KoaRouter = require('koa-router');
var cheerio = require('cheerio');
var fs = require('fs');
var url = require('url');
var server = require('react-dom/server');
var path = require('path');
var path2Regexp = _interopDefault(require('path-to-regexp'));
var send = _interopDefault(require('koa-send'));
var history = require('history');

/*! *****************************************************************************
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */

function __rest(s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) if (e.indexOf(p[i]) < 0)
            t[p[i]] = s[p[i]];
    return t;
}

function __awaiter(thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
}

const routerStore = {
    location: {
        pathname: '',
        search: '',
        state: null,
        hash: '',
    },
};
const RouterContext = React.createContext(routerStore);

const DEFAULT_WEBPACK_SERVER_PORT = 8080;
const DEFAULT_DEV_SERVER_PORT = 3030;
const DEFAULT_CLIENT_PUBLIC_PATH = '/static/';
const DEFAULT_SERVER_PUBLIC_PATH = '/';
const rootDirectory = process.cwd();
function getConfig() {
    console.log('scripts/config -> root directory', rootDirectory);
    const htmlFilename = 'client.html';
    const staticDirName = 'static';
    const buildDirName = 'build';
    return {
        htmlFilename,
        buildDirName,
        staticDirName,
        clientChunkName: 'app.js',
        serverChunkName: 'server.js',
        webpackServerPort: DEFAULT_WEBPACK_SERVER_PORT,
        serverPort: DEFAULT_DEV_SERVER_PORT,
        clientPublicPath: DEFAULT_CLIENT_PUBLIC_PATH,
        serverPublicPath: DEFAULT_SERVER_PUBLIC_PATH,
        htmlTemplatePath: path.join(rootDirectory, 'src/index.html'),
        htmlPath: path.join(rootDirectory, staticDirName, 'client.html'),
        buildDirectory: path.join(rootDirectory, buildDirName),
        staticDirectory: path.join(rootDirectory, staticDirName),
        srcDirectory: path.join(rootDirectory, 'src'),
        customConfigFile: path.join(rootDirectory, 'server-renderer.config.js'),
    };
}

class Router extends React.Component {
    constructor(props) {
        super(props);
        this.fetchInitialProps = (matchedRoute, url) => __awaiter(this, void 0, void 0, function* () {
            if (!matchedRoute) {
                return null;
            }
            const { AppContainer } = this.props;
            if (AppContainer && AppContainer.getInitialProps) {
                this.setState({ loading: true });
                const initialProps = yield AppContainer.getInitialProps({
                    url,
                    Component: matchedRoute.component,
                });
                this.setState({
                    pageProps: initialProps,
                    loading: false,
                });
            }
        });
        this.initHistory = (history$1) => {
            if ('undefined' === typeof window) {
                history$1 = history.createMemoryHistory();
            }
            history$1 = history$1 || history.createBrowserHistory();
            Router.push = history$1.push;
            Router.replace = history$1.replace;
            Router.go = history$1.go;
            Router.goBack = history$1.goBack;
            return history$1;
        };
        const { routes, location: path, pageProps, history: history$1 } = props;
        const location = this.initLocation(path);
        const matchedRoute = this.getMatchedRoute(routes, location.pathname);
        const component = matchedRoute && matchedRoute.component;
        this.history = this.initHistory(history$1);
        this.state = {
            loading: false,
            component,
            pageProps,
            routerStore: {
                location,
            },
        };
    }
    componentDidMount() {
        const { routes } = this.props;
        this.unlisten = this.history.listen((location) => {
            const matched = this.getMatchedRoute(routes, location.pathname);
            const component = matched ? matched.component : null;
            this.setState({
                routerStore: {
                    location,
                },
                component,
                loading: false,
                pageProps: {},
            }, () => {
                this.fetchInitialProps(matched, location.pathname);
            });
        });
    }
    render() {
        const { routerStore } = this.state;
        return (React.createElement(RouterContext.Provider, { value: routerStore }, this.renderContent()));
    }
    componentWillUnmount() {
        this.unlisten();
    }
    renderContent() {
        const { AppContainer, Error } = this.props;
        const { component, pageProps, loading } = this.state;
        if (!component) {
            return (React.createElement(Error, { error: 'Page not found' }));
        }
        return (React.createElement(AppContainer, Object.assign({ loading: loading, Component: component }, pageProps)));
    }
    getMatchedRoute(routes, pathname) {
        const matched = routes.find(route => {
            return path2Regexp(route.path, [], { strict: true }).test(pathname);
        });
        return matched || routes.find(route => route.path === '*' || !route.path);
    }
    initLocation(location) {
        const url$1 = url.parse(location);
        return {
            state: null,
            pathname: url$1.pathname,
            search: url$1.search,
            hash: url$1.hash,
        };
    }
}

const Link = (_a) => {
    var { to, className = '', activeClassName = '', onClick } = _a, restProps = __rest(_a, ["to", "className", "activeClassName", "onClick"]);
    const { location } = React.useContext(RouterContext);
    const matched = path2Regexp(location.pathname).test(to);
    const cls = className + matched ? activeClassName : '';
    const handleClick = (evt) => {
        if (onClick) {
            return onClick(evt);
        }
        if (!evt.isDefaultPrevented()) {
            evt.preventDefault();
            Router.push(to);
        }
    };
    return (React.createElement("a", Object.assign({ href: to, className: cls, onClick: handleClick }, restProps)));
};

const Error = ({ error }) => {
    return (React.createElement("div", { "data-error": true }, String(error)));
};

const config = getConfig();
const isDev = process.env.NODE_ENV === 'development';
class Server {
    constructor(opts) {
        this.port = config.serverPort;
        this.clientChunkPath = new url.URL(config.clientChunkName, `http://localhost:${config.webpackServerPort}${config.clientPublicPath}`);
        this.container = opts.container;
        this.AppContainer = opts.AppContainer || React.Fragment;
        this.Error = opts.Error || Error;
        this.routes = opts.routes;
        const htmlPath = isDev ? config.htmlTemplatePath : config.htmlPath;
        console.log(htmlPath);
        this.originalHTML = fs.readFileSync(htmlPath, 'utf-8');
    }
    start() {
        const app = new Koa();
        const router = new KoaRouter();
        if (!isDev) {
            app.use(this.serveFiles);
        }
        router.get('*', this.handleRequest.bind(this));
        app.use(router.routes());
        app.listen(this.port, () => {
            console.log('Server listen on: http://localhost:' + this.port);
        });
    }
    handleRequest(ctx) {
        return __awaiter(this, void 0, void 0, function* () {
            const routes = this.routes;
            const url = ctx.url;
            const fullUrl = ctx.origin + url;
            const matchedRoute = routes.find(route => {
                return path2Regexp(route.path, [], { strict: true }).test(url);
            });
            const AppContainer = this.AppContainer;
            const Error = this.Error;
            if (!matchedRoute) {
                const content = server.renderToString(React.createElement(Router, { location: fullUrl, AppContainer: AppContainer, Error: Error, pageProps: {}, routes: routes }));
                return ctx.body = this.renderHTML(content, {}, 'Page not found');
            }
            const { pageProps, error } = yield this.getInitialProps(AppContainer, matchedRoute, fullUrl);
            const app = server.renderToString(React.createElement(Router, { location: fullUrl, AppContainer: AppContainer, Error: Error, pageProps: pageProps, routes: routes, error: error }));
            ctx.body = this.renderHTML(app, pageProps, error);
        });
    }
    serveFiles(ctx, next) {
        return __awaiter(this, void 0, void 0, function* () {
            const staticDirName = config.staticDirName;
            const staticDirectory = config.staticDirectory;
            const path = ctx.path;
            if (path.startsWith(`/${staticDirName}/`)) {
                yield send(ctx, path.replace(`/${staticDirName}/`, ''), { root: staticDirectory });
            }
            else {
                yield next();
            }
        });
    }
    renderHTML(content, pageProps, error) {
        const $ = cheerio.load(this.originalHTML, { decodeEntities: false });
        $(this.container).append(content);
        $('head').append(`
      <script type='text/javascript'>
          __APP_DATA__="${encodeURIComponent(JSON.stringify({ pageProps, error }))}"
      </script>
    `);
        if (isDev) {
            $('body').append(`
        <script type='text/javascript' src='${this.clientChunkPath}'></script>
      `);
        }
        return $.html();
    }
    getInitialProps(AppContainer, matchedRoute, url) {
        return __awaiter(this, void 0, void 0, function* () {
            if (AppContainer.getInitialProps) {
                try {
                    const pageProps = yield AppContainer.getInitialProps({
                        Component: matchedRoute.component,
                        url,
                    });
                    return { pageProps, error: null };
                }
                catch (error) {
                    return { pageProps: {}, error };
                }
            }
            return {
                pageProps: {},
                error: null,
            };
        });
    }
}
function render(opts) {
    const server = new Server(opts);
    server.start();
}

Object.keys(history).forEach(function (key) { exports[key] = history[key]; });
exports.render = render;
exports.Router = Router;
exports.Link = Link;
exports.RouterContext = RouterContext;
